Software Design 2022年 1月号 UNIXテキスト処理の極意 第6回 掲載ソースコード
==========================================================================

# ソフトウェアの利用について

スクリプトは、紙面の解説を補い、シェルスクリプティングを用いたテキスト処理の学習を目的として提供するものですが、個人利用、商用利用を問わず、任意に改変するなどして使っていただいて構いません。

ただし、これらのスクリプトおよび関連する技術を用いて、利用者が被る不利益に対して、筆者及び技術評論社は、一切責任を負いません。

# 1. update\_blacklist.bash

2021年12月号掲載 第5回に作成したgeoip.bashを用いて、Apacheへの海外からのアクセスに対するBlack Listを作成するスクリプト

## 起動オプション

<dl>
<dt>a</dt><dd>アクセスログ(access\_log)も検索対象に含める (default:含めない)</dd>
<dt>g</dt><dd>geoipを用いたIPアドレスと地域情報の検索を実施する (default:検索しない)</dd>
<dt>d</dt><dd>Dry Run. 調査結果を出力するのみで、Black Listの更新を行わない (default:更新する)</dd>
</dl>

## 解説
Apacheのログファイル(error\_log, access\_log)を対象として、望まない地域からのアクセスに対して、ブラックリスト(blacklist)を作成します。あらかじめ作成しておくホワイトリスト(whitelist)に記述のあるIPアドレスは、blacklistに含めません。

IPアドレスと地域情報の照合には、geoip.bash(2021年 12月号掲載)が必要です。

スクリプトファイル中17行目の変数prefixに、blacklistとwhitelistを保存するディレクトリを指定します。

blacklist, whitelist とも、1行に1つのIPアドレスを記述する形式です。
実行前に、blacklist, whitelistを(空のファイルでよいので)作成しておきます。

変数prefixが空文字列でなければ、sudoなどによってroot権限でなければ実行できません。デバッグなど、動作確認のためにroot権限でなくても実行する必要がある場合には、prefixを空文字列にするか、24〜26行目をコメントアウトしてください。

Apacheログファイルに含むgeoipで照合対象のIPアドレスが多い場合には、api.geoiplookup.net へのAPI問い合わせ回数が増えますので、リストの作成に相当の時間がかかります。動作確認には、小さなログファイルを作成して対象とするとよいでしょう。Apache2のログディレクトリは、15行目に指定しています。

# 2. readom.bash
XMLなどの \<タグ名\>エレメント\</タグ名\> といった形式のDOMを読み込み、見やすい形式に変換して出力するスクリプト

## 起動オプション
<dl>
<dt>c</dt><dd>タグを色付け表示</dd>
<dt>i</dt><dd>DOMの木構造を階層表示する</ddt>
<dt>e</dt><dd>コメント部分を削除する</dd>
<dt>h</dt><dd>ヘルプを表示</dd>
</dl>

## 解説
変数HB:  30行目で、タグをハイライトするエスケープシーケンスを定義。16行目は、ハイライトしないための「空文字列」
変数HCB: 31行目で、XMLのコメントをハイライトするエスケープシーケンスを定義。17行目は、ハイライトしないための「空文字列」
変数HE:  32行目で、色指定を解除するエスケープシーケンスを定義。18行目は、ハイライトしないときに用いる「空文字列」

変数ts: 35行目で、インデントを何文字にするか設定。20行目では、インデントしないときのために0に設定。

変数CB: XMLコメントの開始にマッチする正規表現
変数CE: XMLコメントの終了にマッチする正規表現

変数ec: コメントを削除する場合のフラグ(0以外なら削除する)

50〜52行目でエレメント内のXMLエスケープ文字の変換と、空行を削除
61〜64行目でタグ内のXMLエスケープ文字の変換と、empty-element ( <タグ名 /> )への対応

## 実行例

図は、yahooニュースのトピックスを記したxmlをcurlで取得し、readomで、インデントとタグの色付けをして表示した例です。

※ 画面上は、タグの部分には色が着いています。
``` readomの実行例
$ curl -s https://news.yahoo.co.jp/rss/topics/top-picks.xml | readom -ic
?xml version="1.0" encoding="UTF-8" standalone="yes"?: 
rss version="2.0": 
  channel: 
    language: ja
    copyright: © Yahoo Japan
    pubDate: 2021-11-21T10:50:20.048Z
    title: Yahoo!ニュース・トピックス - 主要
    link: https://news.yahoo.co.jp/topics/top-picks?source=rss
    description: Yahoo! JAPANのニュース・トピックスで取り上げている最新の見出しを提供しています。
    item: 
      title: 3回目接種へ「混乱」教訓に準備
      link: https://news.yahoo.co.jp/pickup/6410362?source=rss
      pubDate: 2021-11-21T10:01:00.000Z
      description: 新型コロナワクチンの3回目接種が12月から始まるのを前に、自治体は準備を進めている。1、2回目の接種が早かった医療従事者から徐々に始めていくが、予約などを巡り一部地域で混乱を来した今春の教訓を基に、運営方法の改良に努める。
      comments: https://news.yahoo.co.jp/articles/c76f68ed805196fe74e3f38cc2517fa3b801d078/comments
...略
```
