#!/bin/bash

access=0
dry=0
geo=0
while getopts :adg OPT ; do
  case $OPT in
    a) access=1 ;; # include access log
    g) geo=1 ;;    # geo location check
    d) dry=1 ;;    # dry run
  esac
done
shift $((OPTIND - 1))

[ $# -lt 1 ] && logdir="/var/log/apache2/*" && set - ${logdir}

prefix="/usr/local/var/ufw/"
blacklist="blacklist"
whitelist="whitelist"

blacklist="${prefix}${blacklist}"
whitelist="${prefix}${whitelist}"

if [ -n "${prefix}" ]; then
    [ $(whoami) != "root" ] && echo "must execute as root" && exit 1
fi

declare -a acc=()
declare -a err=()

_acc="^access.*"
_err="^error.*"

for f; do
    if [[ ${f##*/} =~ $_err ]]; then
        err+=( $f )
    elif [ $access -gt 0 ]; then
        [[ ${f##*/} =~ $_acc ]] && acc+=( $f )
    fi
done

bl=$(cat ${blacklist})

if [ $geo -eq 0 ]; then # skip geo location check
    [ ${#err[@]} -gt 0 ] &&
    bl=$(echo -e "${bl}\n$(
    xzegrep -o '\[client .*]' "${err[@]}" |
    egrep -o '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}'
    )")
else
    [ ${#err[@]} -gt 0 ] &&
    bl=$(echo -e "${bl}\n$(
    xzegrep -o '\[client .*]' "${err[@]}" |
    egrep -o '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}' |
    xargs geoip -c |
    fgrep -v -e 'Japan' -e 'United States' -e 'Germany' |
    cut -d: -f1
    )")
fi

[ ${#acc[@]} -gt 0 ] &&
bl=$(echo -e "${bl}\n$(
xzegrep -ho '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}' "${acc[@]}" |
xargs geoip -c |
fgrep -v -e 'Japan' -e 'United States' -e 'Germany' |
cut -d: -f1
)")

[ $dry -eq 0 ] && exec > ${blacklist}
comm -23 <( sort <<< ${bl} | uniq )  <(sort ${whitelist}) | sort -t. -n
